@@ -4,27 +4,32 @@ auc.batch <- function(conc, time, group=NULL, method=c("t", "z", "boott"),  alte
   resample <- function(w, conc, time, group, alpha, nsample){
     B<-length(time)
     obsv.parm <- parms(w=w, conc=conc, time=time, group=group)
+
     boot.conc<-vector('list',B)
     boot.group<-vector('list',B)
+    boot.time <-vector('list',B)
     stat <- rep(NA, nsample) 
     for(j in 1:nsample){
        ## generate bootstrap data
        if(!is.null(group)){
          grps<-unique(unlist(group))
-         n<-length(conc[[1]][group[[1]]==grps[1]])/length(time[[1]])
-         m<-length(conc[[1]][group[[1]]==grps[2]])/length(time[[1]])
+         n<-length(conc[[1]][group[[1]]==grps[1]])/length(unique(time[[1]]))
+         m<-length(conc[[1]][group[[1]]==grps[2]])/length(unique(time[[1]]))
          for (i in 1:B){
            boot.conc[[i]]<-c(as.vector(matrix(conc[[i]][group[[i]]==grps[1]],nrow=n)[sample(1:n,replace=TRUE),]),as.vector(matrix(conc[[i]][group[[i]]==grps[2]],nrow=m)[sample(1:m,replace=TRUE),]))
            boot.group[[i]]<-c(rep(1,table(group[[i]])[1]),rep(2,table(group[[i]])[2]))
+           boot.time[[i]] <- c(rep(unique(time[[i]]),each=n),rep(unique(time[[i]]),each=m))
          }
        }else{
-         n<-length(conc[[1]])/length(time[[1]])
+         n<-length(conc[[1]])/length(unique(time[[1]]))
          for (i in 1:B){
            boot.conc[[i]]<-as.vector(matrix(conc[[i]],nrow=n)[sample(1:n,replace=TRUE),])
          }
+         boot.time <- time
          boot.group<-NULL
        }
-       boot.parm <- parms(w=w, conc=boot.conc, time=time, group=boot.group)
+
+       boot.parm <- parms(w=w, conc=boot.conc, time=boot.time, group=boot.group)
        stat[j] <- (boot.parm$est- obsv.parm$est) / sqrt(boot.parm$var)
     }
     lower <- quantile(stat, alpha/2, method=5)
@@ -34,51 +39,108 @@ auc.batch <- function(conc, time, group=NULL, method=c("t", "z", "boott"),  alte
 
   # function to calculate AUC and V(AUC)
   parms <- function(w, conc, time, group){
+
+    single.AUC <- function(w, conc, time){
+
+      concmat <- .batchtomatrix(conc,time)
+      J <- nrow(concmat)
+      ### total number of subjects
+      n.tot <- ncol(concmat)
+      #incidence matrix
+      Inc <-matrix(0,nrow=J,ncol=n.tot)
+      Inc[!is.na(concmat)] <- 1
+      
+      delta <- matrix(NA,ncol=n.tot,nrow=J)
+      for(i in 1:n.tot){
+        delta[,i] <- sum(colSums(Inc[,i]==Inc)==J)/rowSums(Inc)
+      }
+      delta[Inc==0]<- NA
+
+  ## compute AUC
+  # find which batch each observation belongs to
+      unique_batches <- t(unique(t(Inc)))
+      nb <- ncol(unique_batches)
+      b <- rep(NA,n.tot)
+      for( i in 1:nb){
+        b[colSums(Inc==unique_batches[,i])==J] <- i
+      }
+
+      delta[is.na(delta)] <- 0
+      AUC <- sum(tapply(colSums(delta*matrix(1,ncol=n.tot,nrow=J)*w*concmat,na.rm=TRUE),b,mean))  # partial AUC for each individual
+
+      ### number of observation following each batch
+      n <- table(b)
+      R <- Inc %*% t(Inc)
+      rvec <- rowSums(Inc)
+
+      ## average per timepoint
+      y.mean <- rowMeans(concmat, na.rm=TRUE)
+      ## deviation from mean
+      y.dif <- apply(concmat,2,"-",y.mean)
+
+      ## covariance 
+      cov.jh <- matrix(NA,nrow(concmat),nrow(concmat))
+      diag(cov.jh)<- diag(var(t(concmat),na.rm=TRUE,use="pairwise.complete.obs"))
+      for (j1 in 1:(nrow(concmat))) {
+        for(j2 in (j1):nrow(concmat)) {
+          r <- R[j1,j2]
+          cov.jh[j1,j2] <- sum(y.dif[j1,]*y.dif[j2,],na.rm=TRUE)/((r-1)+(1-r/rvec[j1])*(1-r/rvec[j2]))
+          if(!is.nan(cov.jh[j1,j2]) & abs(cov.jh[j1,j2]) > sqrt(cov.jh[j1,j1]*cov.jh[j2,j2])){
+            cov.jh[j1,j2] <- sign(cov.jh[j1,j2])*sqrt(cov.jh[j1,j1]*cov.jh[j2,j2])
+          }
+        }
+      }
+
+      ind <-lower.tri(cov.jh)
+      cov.jh[ind]<-t(cov.jh)[ind]
+
+      cov.jh[is.nan(cov.jh)] <- 0
+      vAUC <- 0
+      temp <- rep(NA,n.tot)
+      for(i in 1:n.tot){
+        help <- 0
+        for(j1 in 1:J){
+          help <- help + sum(delta[j1,i]*delta[,i]*w[j1]*w*cov.jh[j1,])
+        }
+        temp[i] <- 1/n[b[i]]^2*help                
+      }
+      # vector of variance of partial AUCs
+      vpAUC <- tapply(temp,b,sum)
+      vAUC <- sum(vpAUC)
+      df <- vAUC^2/sum(vpAUC^2/((n-1)))
+
+      res <- list(var=vAUC, est=AUC, df=df)
+      return(res)
+
+    }
+
     B<-length(time) ### number of batches
-    tindex<-vector('list',B)
-    rtimes<-rank(unlist(time))
-    h<-1
-    l<-unlist(lapply(time,length))
-    for(i in 1:B){
-      tindex[[i]]<-rtimes[seq(from=h,length=l[i])]
-      h<-h+l[i]
-    } 
+
     ## difference of two AUC's
     if(!is.null(group)){
-      conc1<-vector('list',B)
-      conc2<-vector('list',B)
+ 
+      conc1 <- conc2 <- time1<-time2 <- vector('list',B)
       grps<-unique(unlist(group))
-      n<-length(conc[[1]][group[[1]]==grps[1]])/length(time[[1]])
-      m<-length(conc[[1]][group[[1]]==grps[2]])/length(time[[1]])
-  
+      
       # seperating groups
       for(i in 1:B){
         conc1[[i]]<-conc[[i]][group[[i]]==grps[1]]
+        time1[[i]]<-time[[i]][group[[i]]==grps[1]]
         conc2[[i]]<-conc[[i]][group[[i]]==grps[2]]
+        time2[[i]]<-time[[i]][group[[i]]==grps[2]]
       }
-      if(unlist(length(conc1))/n!=unlist(length(conc2))/m) stop('Number of observations does not match number of time points')
-      auc1<-0
-      auc2<-0
-      aucvar1<-rep(NA,B)
-      aucvar2<-rep(NA,B)
-      for(i in 1:B){
-        auc1<-auc1 +  mean(colSums(w[tindex[[i]]]*matrix(conc1[[i]],ncol=n,byrow=TRUE)))
-        auc2<-auc2 +  mean(colSums(w[tindex[[i]]]*matrix(conc2[[i]],ncol=m,byrow=TRUE)))
-        aucvar1[i]<-sum(w[tindex[[i]]]%o%w[tindex[[i]]]*cov(matrix(conc1[[i]],nrow=n)))/n
-        aucvar2[i]<-sum(w[tindex[[i]]]%o%w[tindex[[i]]]*cov(matrix(conc2[[i]],nrow=m)))/m
-      }
-      res <- list(var=sum(aucvar1)+sum(aucvar2), est=auc1-auc2, df=(sum(aucvar1)/n)^2/sum(aucvar1^2/(n^2*(n-1)))+(sum(aucvar2)/m)^2/sum(aucvar2^2/(m^2*(m-1))))
-      return(res)
-    }else{      
-      n<-length(conc[[1]])/length(time[[1]])
-      auc<-0
-      aucvar<-rep(NA,B)
-      for(i in 1:B){
-        auc<-auc +  mean(colSums(w[tindex[[i]]]*matrix(conc[[i]],ncol=n,byrow=TRUE)))
-        aucvar[i]<-sum(w[tindex[[i]]]%o%w[tindex[[i]]]*cov(matrix(conc[[i]],nrow=n)))/n
-      }
-      res <- list(var=sum(aucvar), est=auc, df=(sum(aucvar)/n)^2/sum(aucvar^2/(n^2*(n-1))))
+      
+      res1 <- single.AUC(w,conc1,time1)
+      res2 <- single.AUC(w,conc2,time2)
+
+      res <- list(var=res1$var+res2$var, est=res1$est-res2$est, df=res1$df+res2$df)
       return(res)
+    }else{
+
+      ## single AUC      
+    
+      return(single.AUC(w,conc,time))
+
     }
   }
 
@@ -114,9 +176,10 @@ auc.batch <- function(conc, time, group=NULL, method=c("t", "z", "boott"),  alte
   if (length(time) != length(conc)) stop('Number of batches defined in time and conc are different')
   if (length(time) != length(group) && !is.null(group)) stop('Number of batches defined in group does not match time and conc')
 
+
   n<-as.numeric(table(time[[1]])[1])
-  if(sum(as.vector(table(unlist(time)))!=rep(n,times=length(unique(unlist(time)))))>0) stop('Number of observations differs between batches')
-  if (sum(table(unlist(lapply(time,unique)))>1)>0) stop('Time points in each batch are not unique')
+#  if(sum(as.vector(table(unlist(time)))!=rep(n,times=length(unique(unlist(time)))))>0) stop('Number of observations differs between batches')
+#  if (sum(table(unlist(lapply(time,unique)))>1)>0) stop('Time points in each batch are not unique')
 
   grpfact <- levels(as.factor(unlist(group)))
   # specify alpha error
@@ -147,13 +210,14 @@ auc.batch <- function(conc, time, group=NULL, method=c("t", "z", "boott"),  alte
   w <- .weight(time.unique)  
 
   # calculate confidence intervals
-  obsv.parm <- parms(w=w, conc=conc, time=time.unique, group=group)
+
+  obsv.parm <- parms(w=w, conc=conc, time=time, group=group)
   if(alternative %in% c('less', 'greater')){alpha <- alpha*2}    
   z <- NULL
   
   method <- sort(method)  
   if (any(method=="boott")){
-    z <- rbind(z,resample(w=w, conc=conc, time=time.unique, group=group, alpha=alpha, nsample=nsample))
+    z <- rbind(z,resample(w=w, conc=conc, time=time, group=group, alpha=alpha, nsample=nsample))
   }
 
   if (any(method=="t")){
